@@ -35,7 +35,11 @@
 
 #include <thunar-tpa/thunar-tpa-bindings.h>
 
-
+#ifdef LIBXFCE4PANEL_CHECK_VERSION
+#if LIBXFCE4PANEL_CHECK_VERSION (4,9,0)
+#define HAS_PANEL_49
+#endif
+#endif
 
 typedef struct _ThunarTpaClass ThunarTpaClass;
 typedef struct _ThunarTpa      ThunarTpa;
@@ -55,6 +59,12 @@ GType           thunar_tpa_get_type            (void);
 void            thunar_tpa_register_type       (XfcePanelTypeModule *type_module);
 static void     thunar_tpa_finalize            (GObject             *object);
 static void     thunar_tpa_construct           (XfcePanelPlugin     *panel_plugin);
+
+#ifdef HAS_PANEL_49
+static gboolean thunar_tpa_size_changed        (XfcePanelPlugin     *panel_plugin,
+                                                gint                 size);
+#endif
+
 static void     thunar_tpa_error               (ThunarTpa           *plugin,
                                                 GError              *error);
 static void     thunar_tpa_state               (ThunarTpa           *plugin,
@@ -147,6 +157,10 @@ thunar_tpa_class_init (ThunarTpaClass *klass)
 
   plugin_class = XFCE_PANEL_PLUGIN_CLASS (klass);
   plugin_class->construct = thunar_tpa_construct;
+
+#ifdef HAS_PANEL_49
+  plugin_class->size_changed = thunar_tpa_size_changed;
+#endif
 }
 
 
@@ -232,6 +246,11 @@ thunar_tpa_construct (XfcePanelPlugin *panel_plugin)
 {
   ThunarTpa *plugin = THUNAR_TPA (panel_plugin);
 
+#ifdef HAS_PANEL_49
+  /* make the plugin fit a single row */
+  xfce_panel_plugin_set_small (panel_plugin, TRUE);
+#endif
+
   /* add the "Empty Trash" menu item */
   xfce_panel_plugin_menu_insert_item (panel_plugin, GTK_MENU_ITEM (plugin->mi));
 
@@ -241,6 +260,23 @@ thunar_tpa_construct (XfcePanelPlugin *panel_plugin)
 
 
 
+#ifdef HAS_PANEL_49
+static gboolean
+thunar_tpa_size_changed (XfcePanelPlugin *panel_plugin,
+                         gint             size)
+{
+  g_return_val_if_fail (panel_plugin != NULL, FALSE);
+
+  /* make the plugin fit a single row */
+  size /= xfce_panel_plugin_get_nrows (panel_plugin);
+  gtk_widget_set_size_request (GTK_WIDGET (panel_plugin), size, size);
+
+  return TRUE;
+}
+#endif
+
+
+
 static void
 thunar_tpa_error (ThunarTpa *plugin,
                   GError    *error)
@@ -403,10 +439,9 @@ thunar_tpa_drag_data_received (GtkWidget        *button,
   if (G_LIKELY (info == TARGET_TEXT_URI_LIST))
     {
       /* check if the data is valid for text/uri-list */
-      if (G_LIKELY (selection_data->length >= 0 && selection_data->format == 8))
+      uri_list = gtk_selection_data_get_uris (selection_data);
+      if (G_LIKELY (uri_list != NULL))
         {
-          /* parse the URI list according to RFC 2483 */
-          uri_list = g_uri_list_extract_uris ((const gchar *) selection_data->data);
           succeed = thunar_tpa_move_to_trash (plugin, (const gchar **) uri_list);
           g_strfreev (uri_list);
         }
