@@ -281,7 +281,7 @@ thunar_chooser_dialog_init (ThunarChooserDialog *dialog)
 
   /* create the "Use as default for this kind of file" button */
   dialog->default_button = gtk_check_button_new_with_mnemonic (_("Use as _default for this kind of file"));
-  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dialog->default_button), TRUE);
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dialog->default_button), FALSE);
   exo_binding_new (G_OBJECT (dialog), "open", G_OBJECT (dialog->default_button), "visible");
   gtk_box_pack_start (GTK_BOX (box), dialog->default_button, FALSE, FALSE, 0);
   gtk_widget_show (dialog->default_button);
@@ -392,7 +392,7 @@ thunar_chooser_dialog_response (GtkDialog *widget,
   GtkTreeIter          iter;
   const gchar         *content_type;
   const gchar         *exec;
-  GAppInfo            *app_info;
+  GAppInfo            *app_info = NULL;
   gboolean             succeed = TRUE;
   GError              *error = NULL;
   gchar               *path;
@@ -456,12 +456,12 @@ thunar_chooser_dialog_response (GtkDialog *widget,
       /* remember the application as default for these kind of file */
       succeed = g_app_info_set_as_default_for_type (app_info, content_type, &error);
 
-      /* verify that we were successfull */
+      /* verify that we were successful */
       if (G_UNLIKELY (!succeed))
         {
           /* display an error to the user */
-          thunar_dialogs_show_error (GTK_WIDGET (dialog), 
-                                     error, 
+          thunar_dialogs_show_error (GTK_WIDGET (dialog),
+                                     error,
                                      _("Failed to set default application for \"%s\""),
                                      thunar_file_get_display_name (dialog->file));
 
@@ -473,6 +473,15 @@ thunar_chooser_dialog_response (GtkDialog *widget,
       if (G_LIKELY (succeed))
         thunar_file_changed (dialog->file);
     }
+  else
+    {
+      /* simply try to set the app as last used for this type (we do not show any errors here) */
+      if (g_app_info_set_as_last_used_for_type (app_info, content_type, NULL))
+        {
+          /* emit "changed" on the file if we successfully changed the default application */
+          thunar_file_changed (dialog->file);
+        }
+    }
 
   /* check if we should also execute the application */
   if (G_LIKELY (succeed && dialog->open))
