@@ -1,4 +1,4 @@
-/* $Id$ */
+/* vi:set et ai sw=2 sts=2 ts=2: */
 /*-
  * Copyright (c) 2005-2011 Benedikt Meurer <benny@xfce.org>
  * Copyright (c) 2009-2011 Jannis Pohlmann <jannis@xfce.org>
@@ -137,8 +137,8 @@ thunar_dialogs_show_rename_file (gpointer    parent,
   /* check if we don't have a directory here */
   if (!thunar_file_is_directory (file))
     {
-      /* check if the filename contains a dot */
-      text = g_utf8_strrchr (filename, -1, '.');
+      /* check if the filename contains an extension */
+      text = thunar_util_str_get_extension (filename);
       if (G_LIKELY (text != NULL))
         {
           /* grab focus to the entry first, else the selection will be altered later */
@@ -519,6 +519,7 @@ thunar_dialogs_show_job_ask_replace (GtkWindow  *parent,
   gchar             *size_string;
   gchar             *text;
   gint               response;
+  gboolean           file_size_binary;
 
   _thunar_return_val_if_fail (parent == NULL || GTK_IS_WINDOW (parent), THUNAR_JOB_RESPONSE_CANCEL);
   _thunar_return_val_if_fail (THUNAR_IS_FILE (src_file), THUNAR_JOB_RESPONSE_CANCEL);
@@ -527,6 +528,7 @@ thunar_dialogs_show_job_ask_replace (GtkWindow  *parent,
   /* determine the style used to format dates */
   preferences = thunar_preferences_get ();
   g_object_get (G_OBJECT (preferences), "misc-date-style", &date_style, NULL);
+  g_object_get (G_OBJECT (preferences), "misc-file-size-binary", &file_size_binary, NULL);
   g_object_unref (G_OBJECT (preferences));
 
   /* setup the confirmation dialog */
@@ -613,7 +615,7 @@ thunar_dialogs_show_job_ask_replace (GtkWindow  *parent,
   g_object_unref (G_OBJECT (icon));
   gtk_widget_show (image);
 
-  size_string = thunar_file_get_size_string (dst_file);
+  size_string = thunar_file_get_size_string_formatted (dst_file, file_size_binary);
   date_string = thunar_file_get_date_string (dst_file, THUNAR_FILE_DATE_MODIFIED, date_style);
   text = g_strdup_printf ("%s %s\n%s %s", _("Size:"), size_string, _("Modified:"), date_string);
   label = gtk_label_new (text);
@@ -644,7 +646,7 @@ thunar_dialogs_show_job_ask_replace (GtkWindow  *parent,
   g_object_unref (G_OBJECT (icon));
   gtk_widget_show (image);
 
-  size_string = thunar_file_get_size_string (src_file);
+  size_string = thunar_file_get_size_string_formatted (src_file, file_size_binary);
   date_string = thunar_file_get_date_string (src_file, THUNAR_FILE_DATE_MODIFIED, date_style);
   text = g_strdup_printf ("%s %s\n%s %s", _("Size:"), size_string, _("Modified:"), date_string);
   label = gtk_label_new (text);
