@@ -347,12 +347,22 @@ thunar_history_error_not_found (GFile    *goto_file,
                                 gpointer  parent)
 {
   gchar  *parse_name;
+  gchar  *path;
   GError *error = NULL;
 
   g_set_error_literal (&error, G_FILE_ERROR, G_FILE_ERROR_EXIST,
                        _("The item will be removed from the history"));
 
-  parse_name = g_file_get_parse_name (goto_file);
+  path = g_file_get_path (goto_file);
+  if (path == NULL)
+    {
+      path = g_file_get_uri (goto_file);
+      parse_name = g_uri_unescape_string (path, NULL);
+      g_free (path);
+    }
+  else
+    parse_name = g_file_get_parse_name (goto_file);
+
   thunar_dialogs_show_error (parent, error, _("Could not find \"%s\""), parse_name);
   g_free (parse_name);
 
@@ -375,7 +385,7 @@ thunar_history_go_back (ThunarHistory  *history,
 
   /* check if the directory still exists */
   directory = thunar_file_get (goto_file, NULL);
-  if (directory == NULL)
+  if (directory == NULL || ! thunar_file_is_mounted (directory))
     {
       thunar_history_error_not_found (goto_file, NULL);
 
@@ -401,7 +411,7 @@ thunar_history_go_back (ThunarHistory  *history,
     }
 
   /* add all the items of the back list to the "forward" list until
-   * the atarget file is reached  */
+   * the target file is reached  */
   for (lp = history->back_list; lp != NULL; lp = lnext)
     {
       lnext = lp->next;
@@ -411,7 +421,7 @@ thunar_history_go_back (ThunarHistory  *history,
           if (directory != NULL)
             history->current_directory = g_object_ref (directory);
 
-          /* remove the new dirctory from the list */
+          /* remove the new directory from the list */
           g_object_unref (lp->data);
           history->back_list = g_slist_delete_link (history->back_list, lp);
 
@@ -456,7 +466,7 @@ thunar_history_go_forward (ThunarHistory  *history,
 
   /* check if the directory still exists */
   directory = thunar_file_get (goto_file, NULL);
-  if (directory == NULL)
+  if (directory == NULL || ! thunar_file_is_mounted (directory))
     {
       thunar_history_error_not_found (goto_file, NULL);
 
@@ -661,7 +671,7 @@ thunar_history_show_menu (GtkAction     *action,
           else if (thunar_g_file_is_root (lp->data))
             icon_name = "drive-harddisk";
           else
-            icon_name = GTK_STOCK_DIRECTORY;
+            icon_name = "folder";
 
           image = gtk_image_new_from_icon_name (icon_name, GTK_ICON_SIZE_MENU);
         }
@@ -740,6 +750,44 @@ thunar_history_set_action_group (ThunarHistory  *history,
 
 
 
+ThunarHistory *
+thunar_history_copy (ThunarHistory  *history,
+                     GtkActionGroup *action_group)
+{
+  ThunarHistory *copy;
+  GSList        *lp;
+
+  _thunar_return_val_if_fail (history == NULL || THUNAR_IS_HISTORY (history), NULL);
+  _thunar_return_val_if_fail (action_group == NULL || GTK_IS_ACTION_GROUP (action_group), NULL);
+
+  if (G_UNLIKELY (history == NULL))
+    return NULL;
+
+  copy = g_object_new (THUNAR_TYPE_HISTORY, NULL);
+
+  /* take a ref on the current directory */
+  copy->current_directory = g_object_ref (history->current_directory);
+
+  /* set the action group */
+  thunar_history_set_action_group (copy, action_group);
+
+  /* copy the back list */
+  for (lp = history->back_list; lp != NULL; lp = lp->next)
+      copy->back_list = g_slist_append (copy->back_list, g_object_ref (G_OBJECT (lp->data)));
+
+  /* copy the forward list */
+  for (lp = history->forward_list; lp != NULL; lp = lp->next)
+      copy->forward_list = g_slist_append (copy->forward_list, g_object_ref (G_OBJECT (lp->data)));
+
+  /* update the sensitivity of the actions */
+  gtk_action_set_sensitive (copy->action_back, (copy->back_list != NULL));
+  gtk_action_set_sensitive (copy->action_forward, (copy->forward_list != NULL));
+
+  return copy;
+}
+
+
+
 /**
  * thunar_file_history_peek_back:
  * @history : a #ThunarHistory.
