@@ -1,4 +1,4 @@
-/* $Id$ */
+/* vi:set et ai sw=2 sts=2 ts=2: */
 /*-
  * Copyright (c) 2004-2007 Benedikt Meurer <benny@xfce.org>
  * Copyright (c) 2009-2011 Jannis Pohlmann <jannis@xfce.org>
@@ -50,6 +50,7 @@ enum
   PROP_FOLDERS_FIRST,
   PROP_NUM_FILES,
   PROP_SHOW_HIDDEN,
+  PROP_FILE_SIZE_BINARY,
   N_PROPERTIES
 };
 
@@ -185,6 +186,7 @@ static gint               thunar_list_model_get_num_files         (ThunarListMod
 static gboolean           thunar_list_model_get_folders_first     (ThunarListModel        *store);
 
 
+
 struct _ThunarListModelClass
 {
   GObjectClass __parent__;
@@ -210,6 +212,7 @@ struct _ThunarListModel
   GSList         *hidden;
   ThunarFolder   *folder;
   gboolean        show_hidden : 1;
+  gboolean        file_size_binary : 1;
   ThunarDateStyle date_style;
 
   /* Use the shared ThunarFileMonitor instance, so we
@@ -328,6 +331,18 @@ thunar_list_model_class_init (ThunarListModelClass *klass)
                             FALSE,
                             EXO_PARAM_READWRITE);
 
+  /**
+   * ThunarListModel::misc-file-size-binary:
+   *
+   * Tells whether to format file size in binary.
+   **/
+  list_model_props[PROP_FILE_SIZE_BINARY] =
+      g_param_spec_boolean ("file-size-binary",
+                            "file-size-binary",
+                            "file-size-binary",
+                            FALSE,
+                            EXO_PARAM_READWRITE);
+
   /* install properties */
   g_object_class_install_properties (gobject_class, N_PROPERTIES, list_model_props);
 
@@ -479,6 +494,10 @@ thunar_list_model_get_property (GObject    *object,
       g_value_set_boolean (value, thunar_list_model_get_show_hidden (store));
       break;
 
+    case PROP_FILE_SIZE_BINARY:
+      g_value_set_boolean (value, thunar_list_model_get_file_size_binary (store));
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -517,6 +536,10 @@ thunar_list_model_set_property (GObject      *object,
       thunar_list_model_set_show_hidden (store, g_value_get_boolean (value));
       break;
 
+    case PROP_FILE_SIZE_BINARY:
+      thunar_list_model_set_file_size_binary (store, g_value_get_boolean (value));
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -716,7 +739,7 @@ thunar_list_model_get_value (GtkTreeModel *model,
 
     case THUNAR_COLUMN_SIZE:
       g_value_init (value, G_TYPE_STRING);
-      g_value_take_string (value, thunar_file_get_size_string (file));
+      g_value_take_string (value, thunar_file_get_size_string_formatted (file, THUNAR_LIST_MODEL (model)->file_size_binary));
       break;
 
     case THUNAR_COLUMN_TYPE:
@@ -1654,6 +1677,12 @@ thunar_list_model_set_case_sensitive (ThunarListModel *store,
 
       /* notify listeners */
       g_object_notify_by_pspec (G_OBJECT (store), list_model_props[PROP_CASE_SENSITIVE]);
+
+      /* emit a "changed" signal for each row, so the display is
+         reloaded with the new case-sensitive setting */
+      gtk_tree_model_foreach (GTK_TREE_MODEL (store),
+                              (GtkTreeModelForeachFunc) gtk_tree_model_row_changed,
+                              NULL);
     }
 }
 
@@ -1858,6 +1887,12 @@ thunar_list_model_set_folders_first (ThunarListModel *store,
   store->sort_folders_first = folders_first;
   g_object_notify_by_pspec (G_OBJECT (store), list_model_props[PROP_FOLDERS_FIRST]);
   thunar_list_model_sort (store);
+
+  /* emit a "changed" signal for each row, so the display is
+     reloaded with the new folders first setting */
+  gtk_tree_model_foreach (GTK_TREE_MODEL (store),
+                          (GtkTreeModelForeachFunc) gtk_tree_model_row_changed,
+                          NULL);
 }
 
 
@@ -1965,6 +2000,63 @@ thunar_list_model_set_show_hidden (ThunarListModel *store,
 
 
 
+/**
+ * thunar_list_model_get_file_size_binary:
+ * @store : a valid #ThunarListModel object.
+ *
+ * Returns %TRUE if the file size should be formatted
+ * as binary.
+ *
+ * Return value: %TRUE if file size format is binary.
+ **/
+gboolean
+thunar_list_model_get_file_size_binary (ThunarListModel *store)
+{
+  _thunar_return_val_if_fail (THUNAR_IS_LIST_MODEL (store), FALSE);
+  return store->file_size_binary;
+}
+
+
+
+/**
+ * thunar_list_model_set_file_size_binary:
+ * @store            : a valid #ThunarListModel object.
+ * @file_size_binary : %TRUE to format file size as binary.
+ *
+ * If @file_size_binary is %TRUE the file size should be
+ * formatted as binary.
+ **/
+void
+thunar_list_model_set_file_size_binary (ThunarListModel *store,
+                                        gboolean         file_size_binary)
+{
+  _thunar_return_if_fail (THUNAR_IS_LIST_MODEL (store));
+
+  /* normalize the setting */
+  file_size_binary = !!file_size_binary;
+
+  /* check if we have a new setting */
+  if (store->file_size_binary != file_size_binary)
+    {
+      /* apply the new setting */
+      store->file_size_binary = file_size_binary;
+
+      /* resort the model with the new setting */
+      thunar_list_model_sort (store);
+
+      /* notify listeners */
+      g_object_notify_by_pspec (G_OBJECT (store), list_model_props[PROP_FILE_SIZE_BINARY]);
+
+      /* emit a "changed" signal for each row, so the display is
+         reloaded with the new binary file size setting */
+      gtk_tree_model_foreach (GTK_TREE_MODEL (store),
+                              (GtkTreeModelForeachFunc) gtk_tree_model_row_changed,
+                              NULL);
+    }
+}
+
+
+
 /**
  * thunar_list_model_get_file:
  * @store : a #ThunarListModel.
@@ -2161,9 +2253,12 @@ thunar_list_model_get_statusbar_text (ThunarListModel *store,
   gint               nrows;
   ThunarPreferences *preferences;
   gboolean           show_image_size;
+  gboolean           file_size_binary;
 
   _thunar_return_val_if_fail (THUNAR_IS_LIST_MODEL (store), NULL);
 
+  file_size_binary = thunar_list_model_get_file_size_binary(store);
+
   if (selected_items == NULL)
     {
       /* try to determine a file for the current folder */
@@ -2176,7 +2271,7 @@ thunar_list_model_get_statusbar_text (ThunarListModel *store,
           && thunar_g_file_get_free_space (thunar_file_get_file (file), &size, NULL)))
         {
           /* humanize the free space */
-          fspace_string = g_format_size (size);
+          fspace_string = g_format_size_full (size, file_size_binary ? G_FORMAT_SIZE_IEC_UNITS : G_FORMAT_SIZE_DEFAULT);
           size_summary = 0;
 
           row = g_sequence_get_begin_iter (store->rows);
@@ -2194,7 +2289,7 @@ thunar_list_model_get_statusbar_text (ThunarListModel *store,
           if (size_summary > 0)
             {
               /* generate a text which includes the size of all items in the folder */
-              size_string = g_format_size (size_summary);
+              size_string = g_format_size_full (size_summary, file_size_binary ? G_FORMAT_SIZE_IEC_UNITS : G_FORMAT_SIZE_DEFAULT);
               text = g_strdup_printf (ngettext ("%d item (%s), Free space: %s", "%d items (%s), Free space: %s", nrows),
                                       nrows, size_string, fspace_string);
               g_free (size_string);
@@ -2231,7 +2326,7 @@ thunar_list_model_get_statusbar_text (ThunarListModel *store,
         }
       else if (G_UNLIKELY (thunar_file_is_symlink (file)))
         {
-          size_string = thunar_file_get_size_string (file);
+          size_string = thunar_file_get_size_string_formatted (file, file_size_binary);
           text = g_strdup_printf (_("\"%s\" (%s) link to %s"), thunar_file_get_display_name (file),
                                   size_string, thunar_file_get_symlink_target (file));
           g_free (size_string);
@@ -2247,7 +2342,7 @@ thunar_list_model_get_statusbar_text (ThunarListModel *store,
       else if (thunar_file_is_regular (file))
         {
           description = g_content_type_get_description (content_type);
-          size_string = thunar_file_get_size_string (file);
+          size_string = thunar_file_get_size_string_formatted (file, file_size_binary);
           /* I18N, first %s is the display name of the file, 2nd the file size, 3rd the content type */
           text = g_strdup_printf (_("\"%s\" (%s) %s"), thunar_file_get_display_name (file),
                                   size_string, description);
@@ -2326,7 +2421,7 @@ thunar_list_model_get_statusbar_text (ThunarListModel *store,
      /* text for the items in the folder */
      if (non_folder_count > 0)
         {
-          size_string = g_format_size (size_summary);
+          size_string = g_format_size_full (size_summary, file_size_binary ? G_FORMAT_SIZE_IEC_UNITS : G_FORMAT_SIZE_DEFAULT);
           if (folder_count > 0)
             {
               /* item count if there are also folders in the selection */
