@@ -1,4 +1,4 @@
-/* $Id$ */
+/* vi:set et ai sw=2 sts=2 ts=2: */
 /*-
  * Copyright (c) 2005-2006 Benedikt Meurer <benny@xfce.org>
  * Copyright (c) 2009 Jannis Pohlmann <jannis@xfce.org>
@@ -35,6 +35,7 @@
 #include <thunar/thunar-icon-factory.h>
 #include <thunar/thunar-location-button.h>
 #include <thunar/thunar-pango-extensions.h>
+#include <thunar/thunar-preferences.h>
 #include <thunar/thunar-private.h>
 
 
@@ -137,7 +138,7 @@ struct _ThunarLocationButton
   ThunarFileIconState file_icon_state;
   
   /* enter folders using DnD */
-  gint                enter_timeout_id;
+  guint               enter_timeout_id;
 
   /* drop support for the button */
   GList              *drop_file_list;
@@ -217,8 +218,9 @@ thunar_location_button_class_init (ThunarLocationButtonClass *klass)
                   G_TYPE_FROM_CLASS (klass),
                   G_SIGNAL_RUN_LAST,
                   0, NULL, NULL,
-                  g_cclosure_marshal_VOID__VOID,
-                  G_TYPE_NONE, 0);
+                  g_cclosure_marshal_VOID__BOOLEAN,
+                  G_TYPE_NONE, 1,
+                  G_TYPE_BOOLEAN);
 
   /**
    * ThunarLocationButton::context-menu:
@@ -263,7 +265,7 @@ thunar_location_button_init (ThunarLocationButton *location_button)
 
   /* create the toggle button */
   button = gtk_toggle_button_new ();
-  GTK_WIDGET_UNSET_FLAGS (button, GTK_CAN_FOCUS);
+  gtk_widget_set_can_focus (button, FALSE);
   g_signal_connect_swapped (G_OBJECT (button), "clicked", G_CALLBACK (thunar_location_button_clicked), location_button);
   exo_mutual_binding_new (G_OBJECT (location_button), "active", G_OBJECT (button), "active");
   gtk_container_add (GTK_CONTAINER (location_button), button);
@@ -424,10 +426,11 @@ thunar_location_button_file_changed (ThunarLocationButton *location_button,
   GtkIconTheme      *icon_theme;
   GtkSettings       *settings;
   GdkPixbuf         *icon;
-  gchar             *icon_name;
+  const gchar       *icon_name;
   gint               height;
   gint               width;
   gint               size;
+  const gchar       *custom_icon;
 
   _thunar_return_if_fail (THUNAR_IS_LOCATION_BUTTON (location_button));
   _thunar_return_if_fail (location_button->file == file);
@@ -479,14 +482,16 @@ thunar_location_button_file_changed (ThunarLocationButton *location_button,
     }
 
   /* setup the DnD icon for the button */
-  icon_name = thunar_file_get_custom_icon (file);
-  if (icon_name == NULL)
+  custom_icon = thunar_file_get_custom_icon (file);
+  if (custom_icon != NULL)
     {
-      icon_name = thunar_file_get_icon_name (file, location_button->file_icon_state, 
-                                             icon_theme);
+      gtk_drag_source_set_icon_name (GTK_BIN (location_button)->child, custom_icon);
+    }
+  else
+    {
+      icon_name = thunar_file_get_icon_name (file, location_button->file_icon_state, icon_theme);
+      gtk_drag_source_set_icon_name (GTK_BIN (location_button)->child, icon_name);
     }
-  gtk_drag_source_set_icon_name (GTK_BIN (location_button)->child, icon_name);
-  g_free (icon_name);
 }
 
 
@@ -570,6 +575,8 @@ thunar_location_button_button_release_event (GtkWidget            *button,
                                              ThunarLocationButton *location_button)
 {
   ThunarApplication *application;
+  ThunarPreferences *preferences;
+  gboolean           open_in_tab;
 
   _thunar_return_val_if_fail (THUNAR_IS_LOCATION_BUTTON (location_button), FALSE);
 
@@ -582,10 +589,22 @@ thunar_location_button_button_release_event (GtkWidget            *button,
       /* verify that we still have a valid file */
       if (G_LIKELY (location_button->file != NULL))
         {
-          /* open a new window for the folder */
-          application = thunar_application_get ();
-          thunar_application_open_window (application, location_button->file, gtk_widget_get_screen (GTK_WIDGET (location_button)), NULL);
-          g_object_unref (G_OBJECT (application));
+          preferences = thunar_preferences_get ();
+          g_object_get (preferences, "misc-middle-click-in-tab", &open_in_tab, NULL);
+          g_object_unref (preferences);
+
+          if (open_in_tab)
+            {
+              /* open in tab */
+              g_signal_emit (G_OBJECT (location_button), location_button_signals[CLICKED], 0, TRUE);
+            }
+          else
+            {
+              /* open a new window for the folder */
+              application = thunar_application_get ();
+              thunar_application_open_window (application, location_button->file, gtk_widget_get_screen (GTK_WIDGET (location_button)), NULL);
+              g_object_unref (G_OBJECT (application));
+            }
         }
     }
 
@@ -634,8 +653,8 @@ thunar_location_button_drag_data_get (GtkWidget            *button,
                                       guint                 timestamp,
                                       ThunarLocationButton *location_button)
 {
-  gchar *uri_string;
-  GList  path_list;
+  gchar **uris;
+  GList   path_list;
 
   _thunar_return_if_fail (GTK_IS_WIDGET (button));
   _thunar_return_if_fail (THUNAR_IS_LOCATION_BUTTON (location_button));
@@ -644,14 +663,13 @@ thunar_location_button_drag_data_get (GtkWidget            *button,
   if (G_LIKELY (location_button->file != NULL))
     {
       /* transform the path into an uri list string */
-      path_list.data = thunar_file_get_file (location_button->file); path_list.next = path_list.prev = NULL;
-      uri_string = thunar_g_file_list_to_string (&path_list);
+      path_list.next = path_list.prev = NULL;
+      path_list.data = thunar_file_get_file (location_button->file);
 
       /* set the uri list for the drag selection */
-      gtk_selection_data_set (selection_data, selection_data->target, 8, (guchar *) uri_string, strlen (uri_string));
-
-      /* cleanup */
-      g_free (uri_string);
+      uris = thunar_g_file_list_to_stringv (&path_list);
+      gtk_selection_data_set_uris (selection_data, uris);
+      g_strfreev (uris);
     }
 }
 
@@ -968,7 +986,7 @@ thunar_location_button_set_file (ThunarLocationButton *location_button,
       g_signal_connect_swapped (G_OBJECT (file), "destroy", G_CALLBACK (thunar_location_button_file_destroy), location_button);
 
       /* update our internal state for the new file (if realized) */
-      if (GTK_WIDGET_REALIZED (location_button))
+      if (gtk_widget_get_realized (GTK_WIDGET (location_button)))
         thunar_location_button_file_changed (location_button, file);
     }
 
@@ -988,7 +1006,7 @@ void
 thunar_location_button_clicked (ThunarLocationButton *location_button)
 {
   _thunar_return_if_fail (THUNAR_IS_LOCATION_BUTTON (location_button));
-  g_signal_emit (G_OBJECT (location_button), location_button_signals[CLICKED], 0);
+  g_signal_emit (G_OBJECT (location_button), location_button_signals[CLICKED], 0, FALSE);
 }
 
 
