@@ -1,4 +1,4 @@
-/* $Id$ */
+/* vi:set et ai sw=2 sts=2 ts=2: */
 /*-
  * Copyright (c) 2005-2006 Benedikt Meurer <benny@xfce.org>
  * Copyright (c) 2009 Jannis Pohlmann <jannis@xfce.org>
@@ -62,7 +62,8 @@ typedef struct _ThunarShortcut ThunarShortcut;
 enum
 {
   PROP_0,
-  PROP_HIDDEN_BOOKMARKS
+  PROP_HIDDEN_BOOKMARKS,
+  PROP_FILE_SIZE_BINARY
 };
 
 
@@ -171,6 +172,7 @@ struct _ThunarShortcutsModel
 
   ThunarPreferences    *preferences;
   gchar               **hidden_bookmarks;
+  gboolean              file_size_binary;
 
   ThunarDeviceMonitor  *device_monitor;
 
@@ -192,7 +194,7 @@ struct _ThunarShortcut
   gint                 sort_id;
 
   guint                busy : 1;
-  guint                busy_pule;
+  guint                busy_pulse;
 
   GFile               *location;
   ThunarFile          *file;
@@ -226,6 +228,19 @@ thunar_shortcuts_model_class_init (ThunarShortcutsModelClass *klass)
                                                        NULL,
                                                        G_TYPE_STRV,
                                                        EXO_PARAM_READWRITE));
+
+  /**
+   * ThunarPropertiesDialog:file_size_binary:
+   *
+   * Whether the file size should be shown in binary or decimal.
+   **/
+  g_object_class_install_property (gobject_class,
+                                   PROP_FILE_SIZE_BINARY,
+                                   g_param_spec_boolean ("file-size-binary",
+                                                         "FileSizeBinary",
+                                                         NULL,
+                                                         FALSE,
+                                                         EXO_PARAM_READWRITE));
 }
 
 
@@ -271,6 +286,10 @@ thunar_shortcuts_model_init (ThunarShortcutsModel *model)
   exo_binding_new (G_OBJECT (model->preferences), "hidden-bookmarks",
                    G_OBJECT (model), "hidden-bookmarks");
 
+  /* binary file size */
+  exo_binding_new (G_OBJECT (model->preferences), "misc-file-size-binary",
+                   G_OBJECT (model), "file-size-binary");
+
   /* load volumes */
   thunar_shortcuts_model_shortcut_devices (model);
 
@@ -302,6 +321,9 @@ thunar_shortcuts_model_finalize (GObject *object)
   g_list_foreach (model->shortcuts, (GFunc) thunar_shortcut_free, model);
   g_list_free (model->shortcuts);
 
+  /* disconnect from the preferences */
+  g_object_unref (model->preferences);
+
   /* free hidden list */
   g_strfreev (model->hidden_bookmarks);
 
@@ -338,6 +360,10 @@ thunar_shortcuts_model_get_property (GObject    *object,
       g_value_set_boxed (value, model->hidden_bookmarks);
       break;
 
+    case PROP_FILE_SIZE_BINARY:
+      g_value_set_boolean (value, model->file_size_binary);
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -394,6 +420,10 @@ thunar_shortcuts_model_set_property (GObject      *object,
       thunar_shortcuts_model_header_visibility (model);
       break;
 
+    case PROP_FILE_SIZE_BINARY:
+      model->file_size_binary = g_value_get_boolean (value);
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -529,6 +559,8 @@ thunar_shortcuts_model_get_value (GtkTreeModel *tree_model,
   gchar          *disk_usage;
   guint32         trash_items;
   gchar          *trash_string;
+  gchar          *parse_name;
+  gboolean        file_size_binary;
 
   _thunar_return_if_fail (iter->stamp == THUNAR_SHORTCUTS_MODEL (tree_model)->stamp);
   _thunar_return_if_fail (THUNAR_IS_SHORTCUTS_MODEL (tree_model));
@@ -582,7 +614,8 @@ thunar_shortcuts_model_get_value (GtkTreeModel *tree_model,
 
           if (file != NULL)
             {
-              disk_usage = thunar_g_file_get_free_space_string (file);
+              file_size_binary = THUNAR_SHORTCUTS_MODEL (tree_model)->file_size_binary;
+              disk_usage = thunar_g_file_get_free_space_string (file, file_size_binary);
               g_object_unref (file);
               g_value_take_string (value, disk_usage);
             }
@@ -614,7 +647,11 @@ thunar_shortcuts_model_get_value (GtkTreeModel *tree_model,
             file = NULL;
 
           if (G_LIKELY (file != NULL))
-            shortcut->tooltip = g_file_get_parse_name (file);
+            {
+              parse_name = g_file_get_parse_name (file);
+              shortcut->tooltip = g_markup_escape_text (parse_name, -1);
+              g_free (parse_name);
+            }
         }
 
       g_value_set_static_string (value, shortcut->tooltip);
@@ -671,7 +708,7 @@ thunar_shortcuts_model_get_value (GtkTreeModel *tree_model,
 
     case THUNAR_SHORTCUTS_MODEL_COLUMN_BUSY_PULSE:
       g_value_init (value, G_TYPE_UINT);
-      g_value_set_uint (value, shortcut->busy_pule);
+      g_value_set_uint (value, shortcut->busy_pulse);
       break;
 
     default:
@@ -890,7 +927,7 @@ thunar_shortcuts_model_shortcut_devices (ThunarShortcutsModel *model)
   shortcut->group = THUNAR_SHORTCUT_GROUP_DEVICES_FILESYSTEM;
   shortcut->name = g_strdup (_("File System"));
   shortcut->file = thunar_file_get_for_uri ("file:///", NULL);
-  shortcut->gicon = g_themed_icon_new (GTK_STOCK_HARDDISK);
+  shortcut->gicon = g_themed_icon_new ("drive-harddisk");
   shortcut->hidden = thunar_shortcuts_model_get_hidden (model, shortcut);
   thunar_shortcuts_model_add_shortcut (model, shortcut);
 
@@ -932,7 +969,7 @@ thunar_shortcuts_model_shortcut_network (ThunarShortcutsModel *model)
   shortcut->group = THUNAR_SHORTCUT_GROUP_NETWORK_DEFAULT;
   shortcut->name = g_strdup (_("Browse Network"));
   shortcut->location = g_file_new_for_uri ("network://");
-  shortcut->gicon = g_themed_icon_new (GTK_STOCK_NETWORK);
+  shortcut->gicon = g_themed_icon_new ("network-workgroup");
   shortcut->hidden = thunar_shortcuts_model_get_hidden (model, shortcut);
   thunar_shortcuts_model_add_shortcut (model, shortcut);
 }
@@ -964,7 +1001,7 @@ thunar_shortcuts_model_shortcut_places (ThunarShortcutsModel *model)
       shortcut = g_slice_new0 (ThunarShortcut);
       shortcut->group = THUNAR_SHORTCUT_GROUP_PLACES_DEFAULT;
       shortcut->file = file;
-      shortcut->gicon = g_themed_icon_new (GTK_STOCK_HOME);
+      shortcut->gicon = g_themed_icon_new ("go-home");
       shortcut->sort_id = 0;
       shortcut->hidden = thunar_shortcuts_model_get_hidden (model, shortcut);
       thunar_shortcuts_model_add_shortcut (model, shortcut);
@@ -1359,6 +1396,7 @@ thunar_shortcuts_model_save (ThunarShortcutsModel *model)
   gchar          *uri;
   GList          *lp;
   GError         *err = NULL;
+  GFile          *parent = NULL;
 
   _thunar_return_if_fail (THUNAR_IS_SHORTCUTS_MODEL (model));
 
@@ -1384,6 +1422,22 @@ thunar_shortcuts_model_save (ThunarShortcutsModel *model)
         }
     }
 
+  /* create folder if it does not exist */
+  parent = g_file_get_parent (model->bookmarks_file);
+  if (!g_file_make_directory_with_parents (parent, NULL, &err))
+    {
+       if (g_error_matches (err, G_IO_ERROR, G_IO_ERROR_EXISTS))
+         {
+           g_clear_error (&err);
+         }
+       else
+         {
+           g_warning ("Failed to create bookmarks folder: %s", err->message);
+           g_error_free (err);
+         }
+    }
+  g_clear_object (&parent);
+
   /* write data to the disk */
   bookmarks_path = g_file_get_path (model->bookmarks_file);
   if (!g_file_set_contents (bookmarks_path, contents->str, contents->len, &err))
@@ -1650,9 +1704,9 @@ thunar_shortcuts_model_busy_timeout (gpointer data)
         continue;
 
       /* loop the pulse of the shortcut */
-      shortcut->busy_pule++;
-      if (shortcut->busy_pule >= SPINNER_NUM_STEPS)
-        shortcut->busy_pule = 0;
+      shortcut->busy_pulse++;
+      if (shortcut->busy_pulse >= SPINNER_NUM_STEPS)
+        shortcut->busy_pulse = 0;
 
       /* generate an iterator for the path */
       GTK_TREE_ITER_INIT (iter, model->stamp, lp);
